# NAME

HRRR Vert Forecast - A Weather Source API

Quarterhourly vertically interpolated forecast data from the HRRR model for latitude/longitude points and Onpoint IDs. Quarterhourly forecast goes out 18 hours and is available in 15 minute increments. The Weather Source HRRR Forecast API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis.    **IMPORTANT:** To authenticate to the Weather Source HRRR Vert Forecast API, each request must contain your API key set to a custom header named `X-API-KEY`. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key `C0W60UOFRML47ytbXk4xlLBfv` to access example locations found in this documentation.      When using the \\\"Try it out\\\" capability in the documentation, the API key should be added by clicking the green \\\"Authorize\\\" button at the top of the page.   

# VERSION

Automatically generated by the [Swagger Codegen](https://github.com/swagger-api/swagger-codegen) project:

- API version: 2.0.0
- Package version: 
- Build package: io.swagger.codegen.v3.generators.scala.ScalaClientCodegen
For more information, please visit [https://developer.weathersourceapis.com/](https://developer.weathersourceapis.com/)

# Requirements

Building the API client library requires [Maven](https://maven.apache.org/) to be installed.

## Installation

To install the API client library to your local Maven repository, simply execute:

```shell
mvn install
```

To deploy it to a remote Maven repository instead, configure the settings of the repository and execute:

```shell
mvn deploy
```

Refer to the [official documentation](https://maven.apache.org/plugins/maven-deploy-plugin/usage.html) for more information.

### Maven users

Add this dependency to your project's POM:

```xml
<dependency>
    <groupId>io.swagger</groupId>
    <artifactId>swagger-scala-client</artifactId>
    <version>1.0.0</version>
    <scope>compile</scope>
</dependency>
```

### Gradle users

Add this dependency to your project's build file:

```groovy
compile "io.swagger:swagger-scala-client:1.0.0"
```

### SBT users

```scala
libraryDependencies += "io.swagger" % "swagger-scala-client" % "1.0.0"
```

## Documentation for API Endpoints

All URIs are relative to *https://hrrrvertforecast.weathersourceapis.com/v2*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*OnpointsApi* | [**onpointsOnpointIdHeightHeightQuarterhoursGet**](OnpointsApi.md#onpointsOnpointIdHeightHeightQuarterhoursGet) | **GET** /onpoints/{onpointId}/height/{height}/quarterhours | The complete vertically interpolated quarterhourly HRRR forecast for an OnPoint™ point.
*OnpointsApi* | [**onpointsOnpointIdHeightHeightQuarterhoursTimestampGet**](OnpointsApi.md#onpointsOnpointIdHeightHeightQuarterhoursTimestampGet) | **GET** /onpoints/{onpointId}/height/{height}/quarterhours/{timestamp} | A quarterhour of vertically interpolated HRRR forecast data for an OnPoint™ point.
*OnpointsApi* | [**onpointsOnpointIdHeightHeightQuarterhoursTimestampStarttimestampEndGet**](OnpointsApi.md#onpointsOnpointIdHeightHeightQuarterhoursTimestampStarttimestampEndGet) | **GET** /onpoints/{onpointId}/height/{height}/quarterhours/{timestampStart},{timestampEnd} | A contiguous range of vertically interpolated quarterhourly HRRR forecast data for an OnPoint™ point.
*PointsApi* | [**pointsLatitudelongitudeHeightHeightQuarterhoursGet**](PointsApi.md#pointsLatitudelongitudeHeightHeightQuarterhoursGet) | **GET** /points/{latitude},{longitude}/height/{height}/quarterhours | The complete vertically interpolated quarterhourly HRRR forecast for a Latitude/Longitude point.
*PointsApi* | [**pointsLatitudelongitudeHeightHeightQuarterhoursTimestampGet**](PointsApi.md#pointsLatitudelongitudeHeightHeightQuarterhoursTimestampGet) | **GET** /points/{latitude},{longitude}/height/{height}/quarterhours/{timestamp} | A quarterhour of vertically interpolated HRRR forecast data for a Latitude/Longitude point.
*PointsApi* | [**pointsLatitudelongitudeHeightHeightQuarterhoursTimestampStarttimestampEndGet**](PointsApi.md#pointsLatitudelongitudeHeightHeightQuarterhoursTimestampStarttimestampEndGet) | **GET** /points/{latitude},{longitude}/height/{height}/quarterhours/{timestampStart},{timestampEnd} | A contiguous range of vertically interpolated quarterhourly HRRR forecast data for a Latitude/Longitude point.

## Documentation for Models

 - [BoundingPoints](BoundingPoints.md)
 - [BoundingPointsInner](BoundingPointsInner.md)
 - [Cape](Cape.md)
 - [Cin](Cin.md)
 - [CldCvr](CldCvr.md)
 - [CountryCode](CountryCode.md)
 - [CountryName](CountryName.md)
 - [DewPt](DewPt.md)
 - [Distance](Distance.md)
 - [Elevation](Elevation.md)
 - [ErrorObj](ErrorObj.md)
 - [FeelsLike](FeelsLike.md)
 - [FieldListHourObj](FieldListHourObj.md)
 - [Fields](Fields.md)
 - [Grid](Grid.md)
 - [HailSize](HailSize.md)
 - [HeatIndex](HeatIndex.md)
 - [HrrrForecastHourArr](HrrrForecastHourArr.md)
 - [HrrrForecastHourObj](HrrrForecastHourObj.md)
 - [HrrrForecastOnpointHourObj](HrrrForecastOnpointHourObj.md)
 - [HrrrForecastPointHourObj](HrrrForecastPointHourObj.md)
 - [Latitude](Latitude.md)
 - [Longitude](Longitude.md)
 - [MslPres](MslPres.md)
 - [OnpointId](OnpointId.md)
 - [OnpointObj](OnpointObj.md)
 - [PointObj](PointObj.md)
 - [Precip](Precip.md)
 - [Pres](Pres.md)
 - [RadSolar](RadSolar.md)
 - [RadSolarDirect](RadSolarDirect.md)
 - [RelHum](RelHum.md)
 - [Snowfall](Snowfall.md)
 - [SnowfallKuchera](SnowfallKuchera.md)
 - [SpcHum](SpcHum.md)
 - [SubdivCode](SubdivCode.md)
 - [SubdivName](SubdivName.md)
 - [Temp](Temp.md)
 - [Timestamp](Timestamp.md)
 - [TimestampEnd](TimestampEnd.md)
 - [TimestampInit](TimestampInit.md)
 - [TimestampRangeObj](TimestampRangeObj.md)
 - [TimestampStart](TimestampStart.md)
 - [Timezone](Timezone.md)
 - [WetBulb](WetBulb.md)
 - [WindChill](WindChill.md)
 - [WindDir](WindDir.md)
 - [WindSpd](WindSpd.md)

## Documentation for Authorization

Authentication schemes defined for the API:
### ApiKeyAuth

- **Type**: API key
- **API key parameter name**: X-API-KEY
- **Location**: HTTP header



# BUILDING YOUR LIBRARY

See the homepage `https://github.com/swagger-api/swagger-codegen` for full details.
But briefly, clone the git repository, build the codegen codebase, set up your build
config file, then run the API build script. You will need git, Java 7 or 8 and Apache
maven 3.0.3 or better already installed.

Your library files will be built under `WWW::MyProjectName`.

          $ git clone https://github.com/swagger-api/swagger-codegen.git
          $ cd swagger-codegen
          $ mvn package
          $ java -jar modules/swagger-codegen-cli/target/swagger-codegen-cli.jar generate \
    -i [URL or file path to JSON swagger API spec] \
    -l akka-scala \
    -c /path/to/config/file.json \
    -o /path/to/output/folder

Bang, all done. Run the `autodoc` script in the `bin` directory to see the API
you just built.

## Author

support@weathersource.com
