/**
 * HRRR Vert Forecast - A Weather Source API
 * Quarterhourly vertically interpolated forecast data from the HRRR model for latitude/longitude points and Onpoint IDs. Quarterhourly forecast goes out 18 hours and is available in 15 minute increments. The Weather Source HRRR Forecast API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis.    **IMPORTANT:** To authenticate to the Weather Source HRRR Vert Forecast API, each request must contain your API key set to a custom header named `X-API-KEY`. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key `C0W60UOFRML47ytbXk4xlLBfv` to access example locations found in this documentation.      When using the \\\"Try it out\\\" capability in the documentation, the API key should be added by clicking the green \\\"Authorize\\\" button at the top of the page.   
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 *
 * NOTE: This class is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the class manually.
 */
package io.swagger.client.api

import java.text.SimpleDateFormat

import java.math.BigDecimal
import java.util.Date
import io.swagger.client.model.errorObj
import io.swagger.client.model.hrrrForecastOnpointHourObj
import io.swagger.client.{ApiInvoker, ApiException}

import com.sun.jersey.multipart.FormDataMultiPart
import com.sun.jersey.multipart.file.FileDataBodyPart

import javax.ws.rs.core.MediaType

import java.io.File
import java.util.Date
import java.util.TimeZone

import scala.collection.mutable.HashMap

import com.wordnik.swagger.client._
import scala.concurrent.Future
import collection.mutable

import java.net.URI

import com.wordnik.swagger.client.ClientResponseReaders.Json4sFormatsReader._
import com.wordnik.swagger.client.RequestWriters.Json4sFormatsWriter._

import scala.concurrent.ExecutionContext.Implicits.global
import scala.concurrent._
import scala.concurrent.duration._
import scala.util.{Failure, Success, Try}

import org.json4s._

class OnpointsApi(
  val defBasePath: String = "https://hrrrvertforecast.weathersourceapis.com/v2",
  defApiInvoker: ApiInvoker = ApiInvoker
) {
  private lazy val dateTimeFormatter = {
    val formatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ")
    formatter.setTimeZone(TimeZone.getTimeZone("UTC"))
    formatter
  }
  private val dateFormatter = {
    val formatter = new SimpleDateFormat("yyyy-MM-dd")
    formatter.setTimeZone(TimeZone.getTimeZone("UTC"))
    formatter
  }
  implicit val formats = new org.json4s.DefaultFormats {
    override def dateFormatter = dateTimeFormatter
  }
  implicit val stringReader: ClientResponseReader[String] = ClientResponseReaders.StringReader
  implicit val unitReader: ClientResponseReader[Unit] = ClientResponseReaders.UnitReader
  implicit val jvalueReader: ClientResponseReader[JValue] = ClientResponseReaders.JValueReader
  implicit val jsonReader: ClientResponseReader[Nothing] = JsonFormatsReader
  implicit val stringWriter: RequestWriter[String] = RequestWriters.StringWriter
  implicit val jsonWriter: RequestWriter[Nothing] = JsonFormatsWriter

  var basePath: String = defBasePath
  var apiInvoker: ApiInvoker = defApiInvoker

  def addHeader(key: String, value: String): mutable.HashMap[String, String] = {
    apiInvoker.defaultHeaders += key -> value
  }

  val config: SwaggerConfig = SwaggerConfig.forUrl(new URI(defBasePath))
  val client = new RestClient(config)
  val helper = new OnpointsApiAsyncHelper(client, config)

  /**
   * The complete vertically interpolated quarterhourly HRRR forecast for an OnPoint™ point.
   * Returns the complete set of vertically interpolated HRRR forecast data for an OnPoint™ point.  *Alternatively using a query that specificies just the hours required for your purposes will improve the query response time.*  **IMPORTANT:** To authenticate to the Weather Source HRRR Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param onpointId An OnPoint™ ID. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall, snowfallKuchera, hailSize)* * allPres *(mslPres, pres)* * allRad *(radSolar, radSolarDirect)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windSpd)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param height The height in meters at which to vertically interpolate. The height MUST be in meters, even if selecting IMPERIAL unit type. 
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return hrrrForecastOnpointHourObj
   */
  def onpointsOnpointIdHeightHeightQuarterhoursGet(onpointId: Integer, fields: List[String], height: BigDecimal, unitScale: Option[String] = Option("IMPERIAL")): Option[hrrrForecastOnpointHourObj] = {
    val await = Try(Await.result(onpointsOnpointIdHeightHeightQuarterhoursGetAsync(onpointId, fields, height, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * The complete vertically interpolated quarterhourly HRRR forecast for an OnPoint™ point. asynchronously
   * Returns the complete set of vertically interpolated HRRR forecast data for an OnPoint™ point.  *Alternatively using a query that specificies just the hours required for your purposes will improve the query response time.*  **IMPORTANT:** To authenticate to the Weather Source HRRR Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param onpointId An OnPoint™ ID. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall, snowfallKuchera, hailSize)* * allPres *(mslPres, pres)* * allRad *(radSolar, radSolarDirect)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windSpd)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param height The height in meters at which to vertically interpolate. The height MUST be in meters, even if selecting IMPERIAL unit type. 
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(hrrrForecastOnpointHourObj)
   */
  def onpointsOnpointIdHeightHeightQuarterhoursGetAsync(onpointId: Integer, fields: List[String], height: BigDecimal, unitScale: Option[String] = Option("IMPERIAL")): Future[hrrrForecastOnpointHourObj] = {
      helper.onpointsOnpointIdHeightHeightQuarterhoursGet(onpointId, fields, height, unitScale)
  }

  /**
   * A quarterhour of vertically interpolated HRRR forecast data for an OnPoint™ point.
   * Returns a single quarterhour of vertically interpolated HRRR forecast data for an OnPoint™ point. The provided timestamp must exist within the current valid 18 hour forecast range.  **IMPORTANT:** To authenticate to the Weather Source HRRR Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param onpointId An OnPoint™ ID. 
   * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall, snowfallKuchera, hailSize)* * allPres *(mslPres, pres)* * allRad *(radSolar, radSolarDirect)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windSpd)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param height The height in meters at which to vertically interpolate. The height MUST be in meters, even if selecting IMPERIAL unit type. 
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return hrrrForecastOnpointHourObj
   */
  def onpointsOnpointIdHeightHeightQuarterhoursTimestampGet(onpointId: Integer, timestamp: Date, fields: List[String], height: BigDecimal, unitScale: Option[String] = Option("IMPERIAL")): Option[hrrrForecastOnpointHourObj] = {
    val await = Try(Await.result(onpointsOnpointIdHeightHeightQuarterhoursTimestampGetAsync(onpointId, timestamp, fields, height, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * A quarterhour of vertically interpolated HRRR forecast data for an OnPoint™ point. asynchronously
   * Returns a single quarterhour of vertically interpolated HRRR forecast data for an OnPoint™ point. The provided timestamp must exist within the current valid 18 hour forecast range.  **IMPORTANT:** To authenticate to the Weather Source HRRR Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param onpointId An OnPoint™ ID. 
   * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall, snowfallKuchera, hailSize)* * allPres *(mslPres, pres)* * allRad *(radSolar, radSolarDirect)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windSpd)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param height The height in meters at which to vertically interpolate. The height MUST be in meters, even if selecting IMPERIAL unit type. 
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(hrrrForecastOnpointHourObj)
   */
  def onpointsOnpointIdHeightHeightQuarterhoursTimestampGetAsync(onpointId: Integer, timestamp: Date, fields: List[String], height: BigDecimal, unitScale: Option[String] = Option("IMPERIAL")): Future[hrrrForecastOnpointHourObj] = {
      helper.onpointsOnpointIdHeightHeightQuarterhoursTimestampGet(onpointId, timestamp, fields, height, unitScale)
  }

  /**
   * A contiguous range of vertically interpolated quarterhourly HRRR forecast data for an OnPoint™ point.
   * Returns a contiguous range of vertically interpolated quarterhourly HRRR forecast data for an OnPoint™ point. The provided timestamps must exist within the current valid 18 hour forecast range.  **IMPORTANT:** To authenticate to the Weather Source HRRR Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param onpointId An OnPoint™ ID. 
   * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. 
   * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall, snowfallKuchera, hailSize)* * allPres *(mslPres, pres)* * allRad *(radSolar, radSolarDirect)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windSpd)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param height The height in meters at which to vertically interpolate. The height MUST be in meters, even if selecting IMPERIAL unit type. 
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return hrrrForecastOnpointHourObj
   */
  def onpointsOnpointIdHeightHeightQuarterhoursTimestampStarttimestampEndGet(onpointId: Integer, timestampStart: Date, timestampEnd: Date, fields: List[String], height: BigDecimal, unitScale: Option[String] = Option("IMPERIAL")): Option[hrrrForecastOnpointHourObj] = {
    val await = Try(Await.result(onpointsOnpointIdHeightHeightQuarterhoursTimestampStarttimestampEndGetAsync(onpointId, timestampStart, timestampEnd, fields, height, unitScale), Duration.Inf))
    await match {
      case Success(i) => Some(await.get)
      case Failure(t) => None
    }
  }

  /**
   * A contiguous range of vertically interpolated quarterhourly HRRR forecast data for an OnPoint™ point. asynchronously
   * Returns a contiguous range of vertically interpolated quarterhourly HRRR forecast data for an OnPoint™ point. The provided timestamps must exist within the current valid 18 hour forecast range.  **IMPORTANT:** To authenticate to the Weather Source HRRR Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
   *
   * @param onpointId An OnPoint™ ID. 
   * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. 
   * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. 
   * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, snowfall, snowfallKuchera, hailSize)* * allPres *(mslPres, pres)* * allRad *(radSolar, radSolarDirect)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windSpd)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
   * @param height The height in meters at which to vertically interpolate. The height MUST be in meters, even if selecting IMPERIAL unit type. 
   * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
   * @return Future(hrrrForecastOnpointHourObj)
   */
  def onpointsOnpointIdHeightHeightQuarterhoursTimestampStarttimestampEndGetAsync(onpointId: Integer, timestampStart: Date, timestampEnd: Date, fields: List[String], height: BigDecimal, unitScale: Option[String] = Option("IMPERIAL")): Future[hrrrForecastOnpointHourObj] = {
      helper.onpointsOnpointIdHeightHeightQuarterhoursTimestampStarttimestampEndGet(onpointId, timestampStart, timestampEnd, fields, height, unitScale)
  }

}

class OnpointsApiAsyncHelper(client: TransportClient, config: SwaggerConfig) extends ApiClient(client, config) {

  def onpointsOnpointIdHeightHeightQuarterhoursGet(onpointId: Integer,
    fields: List[String],
    height: BigDecimal,
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[hrrrForecastOnpointHourObj]): Future[hrrrForecastOnpointHourObj] = {
    // create path and map variables
    val path = (addFmt("/onpoints/{onpointId}/height/{height}/quarterhours")
      replaceAll("\\{" + "onpointId" + "\\}", onpointId.toString)
      replaceAll("\\{" + "height" + "\\}", height.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling OnpointsApi->onpointsOnpointIdHeightHeightQuarterhoursGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }

  def onpointsOnpointIdHeightHeightQuarterhoursTimestampGet(onpointId: Integer,
    timestamp: Date,
    fields: List[String],
    height: BigDecimal,
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[hrrrForecastOnpointHourObj]): Future[hrrrForecastOnpointHourObj] = {
    // create path and map variables
    val path = (addFmt("/onpoints/{onpointId}/height/{height}/quarterhours/{timestamp}")
      replaceAll("\\{" + "onpointId" + "\\}", onpointId.toString)
      replaceAll("\\{" + "timestamp" + "\\}", timestamp.toString)
      replaceAll("\\{" + "height" + "\\}", height.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling OnpointsApi->onpointsOnpointIdHeightHeightQuarterhoursTimestampGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }

  def onpointsOnpointIdHeightHeightQuarterhoursTimestampStarttimestampEndGet(onpointId: Integer,
    timestampStart: Date,
    timestampEnd: Date,
    fields: List[String],
    height: BigDecimal,
    unitScale: Option[String] = Option("IMPERIAL")
    )(implicit reader: ClientResponseReader[hrrrForecastOnpointHourObj]): Future[hrrrForecastOnpointHourObj] = {
    // create path and map variables
    val path = (addFmt("/onpoints/{onpointId}/height/{height}/quarterhours/{timestampStart},{timestampEnd}")
      replaceAll("\\{" + "onpointId" + "\\}", onpointId.toString)
      replaceAll("\\{" + "timestampStart" + "\\}", timestampStart.toString)
      replaceAll("\\{" + "timestampEnd" + "\\}", timestampEnd.toString)
      replaceAll("\\{" + "height" + "\\}", height.toString))

    // query params
    val queryParams = new mutable.HashMap[String, String]
    val headerParams = new mutable.HashMap[String, String]

    if (fields == null) throw new Exception("Missing required parameter 'fields' when calling OnpointsApi->onpointsOnpointIdHeightHeightQuarterhoursTimestampStarttimestampEndGet")
    queryParams += "fields" -> fields.toString
    unitScale match {
      case Some(param) => queryParams += "unitScale" -> param.toString
      case _ => queryParams
    }

    val resFuture = client.submit("GET", path, queryParams.toMap, headerParams.toMap, "")
    resFuture flatMap { resp =>
      process(reader.read(resp))
    }
  }


}
